@@ -1,13 +1,13 @@
 """Mixin for algebraic modules. These are used to calculate e.g. QSSA assumptions."""
+from __future__ import annotations
 
-# Standard Library
 import warnings
 from dataclasses import dataclass, field
+from typing import Union
 
-# Third party
 import numpy as np
+import libsbml
 
-# Local code
 from .utils import (
     get_formatted_function_source_code,
     patch_lambda_function_name,
@@ -33,8 +33,7 @@ class AlgebraicMixin:
     compounds that are calculated before the rate functions are calculated.
     """
 
-    def __init__(self, algebraic_modules=None):
-        self.derived_compounds = []
+    def __init__(self, algebraic_modules: dict = None):
         self.algebraic_modules = {}
         if algebraic_modules is not None:
             self.add_algebraic_modules(algebraic_modules=algebraic_modules)
@@ -43,78 +42,23 @@ class AlgebraicMixin:
     # Derived compound functions
     ##########################################################################
 
-    def _add_derived_compound(self, *, compound):
-        """Add a derived compound to the model.
+    def get_derived_compounds(self) -> list[str]:
+        """Return names of compounds derived from algebraic modules."""
+        derived_compounds = []
+        for module in self.algebraic_modules.values():
+            derived_compounds.extend(module["derived_compounds"])
+        return derived_compounds
 
-        Derived compounds are dynamic quantities calculated by algebraic modules.
-        They are accessible by other rates but are not returned to any integrators.
+    @property
+    def derived_compounds(self) -> list[str]:
+        """Return names of compounds derived from algebraic modules
 
-        Parameters
-        ----------
-        compound : str
-            Name / id of the compound
+        Used to be an attribute of the model, so this is kept to ensure backwards compatability
         """
-        if not isinstance(compound, str):
-            raise TypeError("The compound name should be string")
-        if compound == "time":
-            raise KeyError("time is a protected variable for time")
-        if compound in self.derived_compounds:
-            warnings.warn(f"Overwriting derived compound {compound}")
-        self.derived_compounds.append(compound)
+        return self.get_derived_compounds()
 
-    def _add_derived_compounds(self, *, compounds):
-        """Add multiple derived compounds to the model.
-
-        Derived compounds are dynamic quantities calculated by algebraic modules.
-        They are accessible by other rates but are not returned to any integrators.
-
-        Parameters
-        ----------
-        compounds : Iterable(str)
-
-        See Also
-        --------
-        _add_derived_compound
-        """
-        for compound in compounds:
-            self._add_derived_compound(compound=compound)
-
-    def _remove_derived_compound(self, *, compound):
-        """Remove a derived compound from the model.
-
-        Parameters
-        ----------
-        compound : str
-            Name / id of the compound
-        """
-        self.derived_compounds.remove(compound)
-
-    def _remove_derived_compounds(self, *, compounds):
-        """Remove multiple derived compounds from the model.
-
-        Parameters
-        ----------
-        compounds : Iterable(str)
-        """
-        for compound in compounds:
-            self._remove_derived_compound(compound=compound)
-
-    def get_derived_compounds(self):
-        """Return names of compounds derived from algebraic modules.
-
-        Returns
-        -------
-        derived_compounds : list(str)
-        """
-        return list(self.derived_compounds)
-
-    def get_all_compounds(self):
-        """Return names of compounds and derived compounds in that order.
-
-        Returns
-        -------
-        all_compounds: list(str)
-        """
+    def get_all_compounds(self) -> list[str]:
+        """Return names of compounds and derived compounds (in that order)."""
         return list(self.get_compounds() + self.get_derived_compounds())
 
     ##########################################################################
@@ -123,13 +67,13 @@ class AlgebraicMixin:
 
     def add_algebraic_module(
         self,
-        module_name,
-        function,
-        compounds=None,
-        derived_compounds=None,
-        modifiers=None,
-        parameters=None,
-        args=None,
+        module_name: str,
+        function: callable,
+        compounds: list[str] = None,
+        derived_compounds: list[str] = None,
+        modifiers: list[str] = None,
+        parameters: list[str] = None,
+        args: list[str] = None,
         **meta_info,
     ):
         """Add an algebraic module to the model.
@@ -138,24 +82,6 @@ class AlgebraicMixin:
         The Python function will get the function arguments in the following order:
         [**compounds, **modifiers, **parameters]
 
-        Parameters
-        ----------
-        module_name : str
-            Name of the module
-        function : callable
-            Python method of the algebraic module
-        compounds : iterable(str)
-            Names of compounds used for module
-        derived_compounds : iterable(str)
-            Names of compounds which are calculated by the module
-        modifiers : iterable(str)
-            Names of compounds which act as modifiers on the module
-        parameters : iterable(str)
-            Names of the parameters which are passed to the function
-        meta_info : dict, optional
-            Meta info of the algebraic module. Allowed keys are
-            {common_name, notes, database_links}
-
         Warns
         -----
         UserWarning
@@ -187,12 +113,20 @@ class AlgebraicMixin:
         if args is None:
             args = compounds + modifiers + parameters
 
-        patch_lambda_function_name(function=function, name=module_name)
-
         if module_name in self.algebraic_modules:
             self.remove_algebraic_module(module_name=module_name)
             warnings.warn(f"Overwriting algebraic module {module_name}")
 
+        self._check_for_existence(
+            name=module_name, check_against=self.get_all_compounds(), candidates=compounds + modifiers
+        )
+        self._check_for_existence(
+            name=module_name, check_against=self.get_parameter_names(), candidates=parameters
+        )
+        self._check_and_insert_ids(derived_compounds)
+
+        patch_lambda_function_name(function=function, name=module_name)
+
         self.algebraic_modules[module_name] = {
             "function": function,
             "compounds": compounds,
@@ -201,59 +135,32 @@ class AlgebraicMixin:
             "parameters": parameters,
             "args": args,
         }
-        for compound in derived_compounds:
-            self._add_derived_compound(compound=compound)
 
         self.meta_info.setdefault("modules", {}).setdefault(module_name, Module(**meta_info))
 
-    def add_algebraic_modules(self, algebraic_modules, meta_info=None):
+    def add_algebraic_modules(self, algebraic_modules: dict, meta_info: dict = None):
         """Add multiple algebraic modules to the model.
 
-        CAUTION: The Python function of the module has to return an iterable.
-
-        Parameters
-        ----------
-        algebraic_modules : dict
-
         See Also
         --------
         add_algebraic_module
         """
         meta_info = {} if meta_info is None else meta_info
         for module_name, module in algebraic_modules.items():
-            try:
-                info = meta_info[module_name]
-            except KeyError:
-                info = {}
+            info = meta_info.get(module_name, {})
             self.add_algebraic_module(module_name=module_name, **module, **info)
 
     def update_algebraic_module(
         self,
-        module_name,
-        function=None,
-        compounds=None,
-        derived_compounds=None,
-        modifiers=None,
-        parameters=None,
-        args=None,
+        module_name: str,
+        function: callable = None,
+        compounds: list[str] = None,
+        derived_compounds: list[str] = None,
+        modifiers: list[str] = None,
+        parameters: list[str] = None,
+        args: list[str] = None,
     ):
-        """Update an existing reaction.
-
-        Parameters
-        ----------
-        module_name : str
-            Name of the module
-        function : callable
-            Python method of the algebraic module
-        compounds : iterable(str)
-            Names of compounds used for module
-        derived_compounds : iterable(str)
-            Names of compounds which are calculated by the module
-        modifiers : iterable(str)
-            Names of compounds which act as modifiers on the module
-        parameters : iterable(str)
-            Names of the parameters which are passed to the function
-        """
+        """Update an existing reaction."""
         module = self.algebraic_modules[module_name]
         args_have_changed = False
         if function is not None:
@@ -262,37 +169,47 @@ class AlgebraicMixin:
         if compounds is not None:
             module["compounds"] = compounds
             args_have_changed = True
+        else:
+            compounds = module["compounds"]
         if derived_compounds is not None:
-            old_compounds = module["derived_compounds"]
-
-            # If there are now less derived compounds, remove the excess ones
-            if len(derived_compounds) < len(old_compounds):
-                for compound in old_compounds[len(derived_compounds) :]:
-                    self.derived_compounds.remove(compound)
-
-            # Update old derived compounds (keeping their order)
-            replacements = dict(zip(old_compounds, derived_compounds))
-            self.derived_compounds = [replacements.get(x, x) for x in self.derived_compounds]
-
-            # Add new derived compounds (if there are any)
-            for compound in derived_compounds[len(replacements) :]:
-                self._add_derived_compound(compound=compound)
-
+            derived_changes = dict(zip(module["derived_compounds"], derived_compounds))
+            self._update_ids(derived_changes)
             module["derived_compounds"] = derived_compounds
             args_have_changed = True
         if modifiers is not None:
             module["modifiers"] = modifiers
             args_have_changed = True
+        else:
+            modifiers = module["modifiers"]
         if parameters is not None:
             module["parameters"] = parameters
             args_have_changed = True
+        else:
+            parameters = module["parameters"]
+
+        self._check_for_existence(
+            name=module_name, check_against=self.get_all_compounds(), candidates=compounds + modifiers
+        )
+        self._check_for_existence(
+            name=module_name, check_against=self.get_parameter_names(), candidates=parameters
+        )
+
         if args is not None:
             module["args"] = args
-        else:
-            if args_have_changed:
-                module["args"] = module["compounds"] + module["modifiers"] + module["parameters"]
+        elif args_have_changed:
+            module["args"] = compounds + modifiers + parameters
 
-    def update_module_meta_info(self, module, meta_info):
+    def update_algebraic_modules(self, modules: dict):
+        """Update multiple algebraic modules
+
+        See Also
+        --------
+        update_algebraic_module
+        """
+        for name, module in modules.items():
+            self.update_algebraic_module(name, **module)
+
+    def update_module_meta_info(self, module: str, meta_info: dict):
         """Update meta info of an algebraic module.
 
         Parameters
@@ -305,7 +222,7 @@ class AlgebraicMixin:
         """
         self.update_meta_info(component="modules", meta_info={module: meta_info})
 
-    def remove_algebraic_module(self, module_name):
+    def remove_algebraic_module(self, module_name: str):
         """Remove an algebraic module.
 
         Parameters
@@ -314,10 +231,9 @@ class AlgebraicMixin:
             Name of the algebraic module
         """
         module = self.algebraic_modules.pop(module_name)
-        for compound in module["derived_compounds"]:
-            self._remove_derived_compound(compound=compound)
+        self._remove_ids(module["derived_compounds"])
 
-    def remove_algebraic_modules(self, module_names):
+    def remove_algebraic_modules(self, module_names: list[str]):
         """Remove multiple algebraic modules.
 
         Parameters
@@ -328,78 +244,43 @@ class AlgebraicMixin:
         for module_name in module_names:
             self.remove_algebraic_module(module_name=module_name)
 
-    def get_algebraic_module_compounds(self, module_name):
-        """Return the compounds of the algebraic module.
+    def get_algebraic_module(self, module_name: str) -> dict:
+        return self.algebraic_modules[module_name]
 
-        Parameters
-        ----------
-        module_name : str
-            Name of the algebraic module
+    def get_algebraic_module_function(self, module_name: str) -> callable:
+        return self.algebraic_modules[module_name]["function"]
 
-        Returns
-        -------
-        module_compounds : list(str)
-        """
+    def get_algebraic_module_compounds(self, module_name: str) -> list[str]:
         return list(self.algebraic_modules[module_name]["compounds"])
 
-    def get_algebraic_module_derived_compounds(self, module_name):
-        """Return the derived compounds of the algebraic module.
-
-        Parameters
-        ----------
-        module_name : str
-            Name of the algebraic module
-
-        Returns
-        -------
-        module_compounds : list(str)
-            Derived compounds of the algebraic module
-        """
+    def get_algebraic_module_derived_compounds(self, module_name: str) -> list[str]:
         return list(self.algebraic_modules[module_name]["derived_compounds"])
 
-    def get_algebraic_module_modifiers(self, module_name):
-        """Return the derived compounds of the algebraic module.
-
-        Parameters
-        ----------
-        module_name : str
-            Name of the algebraic module
-
-        Returns
-        -------
-        module_compounds : list(str)
-            Derived compounds of the algebraic module
-        """
+    def get_algebraic_module_modifiers(self, module_name: str) -> list[str]:
         return list(self.algebraic_modules[module_name]["modifiers"])
 
-    def get_algebraic_module_parameters(self, module_name):
-        """Return the parameters of the algebraic module.
-
-        Parameters
-        ----------
-        module_name : str
-            Name of the algebraic module
-
-        Returns
-        -------
-        parameters : list(str)
-            Parameters of the algebraic module
-        """
+    def get_algebraic_module_parameters(self, module_name: str) -> list[str]:
         return list(self.algebraic_modules[module_name]["parameters"])
 
+    def get_algebraic_module_args(self, module_name: str) -> list[str]:
+        return list(self.algebraic_modules[module_name]["args"])
+
     ##########################################################################
     # Simulation functions
     ##########################################################################
 
-    def _get_fcd(self, *, t, y):
+    def _get_fcd(
+        self,
+        *,
+        t: Union[float, list[float]],
+        y: Union[dict[str, float], dict[str, list[float]]],
+    ) -> Union[dict[str, float], dict[str, list[float]]]:
         """Calculate the derived variables of all algebraic modules.
 
-        Parameters
-        ----------
-        t : num, array(num)
-            One are multiple time points
-        y : dict(str: num)
-            A dictionary of the concentrations of all non-derived compounds
+        fdc = full_concentration_dict
+
+        The derived compounds are sorted by the algebraic modules and their internal
+        derived_modules attribute.
         """
         y["time"] = t
         # args = self.parameters | y # that is python 3.9+ syntax, wait for a bit
@@ -417,14 +298,13 @@ class AlgebraicMixin:
             )
             y.update(derived_compounds)
             args.update(derived_compounds)
-        # return dict(set(args) - set(self.parameters)) # alternative solution, think about it
         return y
 
     ##########################################################################
     # Source code functions
     ##########################################################################
 
-    def _generate_algebraic_modules_source_code(self, *, include_meta_info=True):
+    def _generate_algebraic_modules_source_code(self, *, include_meta_info=True) -> tuple[str]:
         """Generate modelbase source code for algebraic modules.
 
         This is mainly used for the generate_model_source_code function.
@@ -453,6 +333,7 @@ class AlgebraicMixin:
             derived_compounds = module["derived_compounds"]
             modifiers = module["modifiers"]
             parameters = module["parameters"]
+            args = module["args"]
 
             function_code = get_formatted_function_source_code(
                 function_name=name, function=function, function_type="module"
@@ -466,6 +347,7 @@ class AlgebraicMixin:
                 f"    derived_compounds={derived_compounds},\n"
                 f"    modifiers={modifiers},\n"
                 f"    parameters={parameters},\n"
+                f"    args={args},\n"
             )
             if include_meta_info:
                 meta_info = self._get_nonzero_meta_info(component="modules")
@@ -482,7 +364,7 @@ class AlgebraicMixin:
     # SBML functions
     ##########################################################################
 
-    def _create_sbml_algebraic_modules(self, *, sbml_model):
+    def _create_sbml_algebraic_modules(self, *, sbml_model: libsbml.Model):
         """Convert the algebraic modules their sbml equivalent.
 
         Notes
