@@ -1,16 +1,15 @@
 """LabelModel base class."""
+from __future__ import annotations
 
-# Standard Library
 import itertools as it
 import warnings
 from collections import defaultdict as _defaultdict
+from typing import Union
 
-# Third party
 import libsbml
 import numpy as np
 import pandas as pd
 
-# Local code
 from ...core.compoundmixin import Compound
 from ...core.ratemixin import Rate
 from ...core.utils import convert_id_to_sbml, warning_on_one_line
@@ -33,12 +32,12 @@ class LabelModel(Model):
 
     def __init__(
         self,
-        parameters=None,
-        compounds=None,
-        algebraic_modules=None,
-        rate_stoichiometries=None,
-        rates=None,
-        meta_info=None,
+        parameters: dict = None,
+        compounds: dict = None,
+        algebraic_modules: dict = None,
+        rate_stoichiometries: dict = None,
+        rates: dict = None,
+        meta_info: dict = None,
     ):
         self.label_compounds = {}
         self.nonlabel_compounds = []
@@ -53,14 +52,9 @@ class LabelModel(Model):
         )
 
     @staticmethod
-    def _generate_binary_labels(*, base_name, num_labels):
+    def _generate_binary_labels(*, base_name: str, num_labels: int) -> list[str]:
         """Create binary label string.
 
-        Parameters
-        ----------
-        base_name : str
-        num_labels : int
-
         Returns
         -------
         isotopomers : list(str)
@@ -82,17 +76,17 @@ class LabelModel(Model):
         else:
             return [base_name]
 
-    def add_compound(self, compound, is_isotopomer=False, **meta_info):
+    def add_compound(self, compound: str, is_isotopomer=False, **meta_info):
         """Add a single compound to the model.
 
         Parameters
         ----------
-        compound : str
+        compound
             Name / id of the compound
-        is_isotopomer : bool
+        is_isotopomer
             Whether the compound is an isotopomer of a base compound
             or a non-label compound
-        meta_info : dict, optional
+        meta_info
             Meta info of the compound. Available keys are
             {common_name, compartment, formula, charge, gibbs0, smiles, database_links, notes}
         """
@@ -100,15 +94,16 @@ class LabelModel(Model):
         if not is_isotopomer:
             self.nonlabel_compounds.append(compound)
 
-    def _add_base_compound(self, *, base_compound, num_labels, label_names, **meta_info):
+    def _add_base_compound(self, *, base_compound: str, num_labels: int, label_names: list[str], **meta_info):
         """Add a base compound of label isotopomer."""
         self.label_compounds[base_compound] = {
             "num_labels": num_labels,
             "isotopomers": label_names,
         }
+        self._check_and_insert_ids([base_compound])
         self.meta_info.setdefault("compounds", {}).setdefault(base_compound, Compound(**meta_info))
 
-    def _add_isotopomers(self, *, base_compound, label_names):
+    def _add_isotopomers(self, *, base_compound: str, label_names: list[str]):
         # Add all labelled compounds
         for compound in label_names:
             self.add_compound(compound=compound, is_isotopomer=True)
@@ -124,7 +119,7 @@ class LabelModel(Model):
             parameters=None,
         )
 
-    def add_label_compound(self, compound, num_labels, **meta_info):
+    def add_label_compound(self, compound: str, num_labels: int, **meta_info):
         """Create all label isotopomers and add them as compounds.
 
         Also create an algebraic module that tracks the total
@@ -132,9 +127,9 @@ class LabelModel(Model):
 
         Parameters
         ----------
-        base_compound : str
+        base_compound
             Base name of the compound
-        num_labels : int
+        num_labels
             Number of labels in the compound
 
         Warns
@@ -157,34 +152,35 @@ class LabelModel(Model):
             )
             self._add_isotopomers(base_compound=compound, label_names=label_names)
 
-    def add_label_compounds(self, compounds, meta_info=None):
+    def add_label_compounds(self, compounds: dict[str, int], meta_info=None):
         """Add multiple label-containing compounds to the model.
 
         Parameters
         ----------
-        compounds : dict(str, int)
-            {compound: num_labels} dictionary
+        compounds
+            {compound: num_labels}
 
         Examples
         --------
         >>> add_label_compounds({"GAP": 3, "DHAP": 3, "FBP": 6})
+
+        See Also
+        --------
+        add_label_compound
         """
         meta_info = {} if meta_info is None else meta_info
 
         for compound, num_labels in compounds.items():
-            try:
-                info = meta_info[compound]
-            except KeyError:
-                info = {}
+            info = meta_info.get(compound, {})
             self.add_label_compound(compound=compound, num_labels=num_labels, **info)
 
-    def remove_compound(self, compound, is_isotopomer=False):
+    def remove_compound(self, compound: str, is_isotopomer=False):
         """Remove a compound from the model.
 
         Parameters
         ----------
-        compound : str
-        is_isotopomer : bool
+        compound
+        is_isotopomer
             Whether the compound is an isotopomer of a base compound
             or a non-label compound
         """
@@ -192,75 +188,32 @@ class LabelModel(Model):
         if not is_isotopomer:
             self.nonlabel_compounds.remove(compound)
 
-    def remove_label_compound(self, compound):
-        """Remove a label compound from the model.
-
-        Parameters
-        ----------
-        compound : str
-            Name of the compound
-        """
+    def remove_label_compound(self, compound: str):
+        """Remove a label compound from the model."""
         label_compound = self.label_compounds.pop(compound)
+        self._remove_ids([compound])
         for key in label_compound["isotopomers"]:
             self.remove_compound(compound=key, is_isotopomer=True)
 
-    def remove_label_compounds(self, compounds):
-        """Remove label compounds.
-
-        Parameters
-        ----------
-        compounds : iterable(str)
-            Names of the compounds
-        """
+    def remove_label_compounds(self, compounds: list[str]):
+        """Remove label compounds."""
         for compound in compounds:
             self.remove_label_compound(compound=compound)
 
-    def get_base_compounds(self):
-        """Get all base compounds and non-label compounds.
-
-        Returns
-        -------
-        base_compounds : list(str)
-        """
+    def get_base_compounds(self) -> list[str]:
+        """Get all base compounds and non-label compounds."""
         return sorted(list(self.label_compounds) + self.nonlabel_compounds)
 
-    def get_compound_number_of_label_positions(self, compound):
-        """Get the number of possible labels positions of a compound.
-
-        Parameters
-        ----------
-        compound : str
-
-        Returns
-        -------
-        num_labels : int
-        """
+    def get_compound_number_of_label_positions(self, compound: str) -> int:
+        """Get the number of possible labels positions of a compound."""
         return int(self.label_compounds[compound]["num_labels"])
 
-    def get_compound_isotopomers(self, compound):
-        """Get all isotopomers of a compound.
-
-        Parameters
-        ----------
-        compound : str
-
-        Returns
-        -------
-        isotopomers : list(str)
-        """
+    def get_compound_isotopomers(self, compound: str) -> list[str]:
+        """Get all isotopomers of a compound."""
         return list(self.label_compounds[compound]["isotopomers"])
 
-    def get_compound_isotopomers_with_n_labels(self, compound, n_labels):
-        """Get all isotopomers of a compound, that have excactly n labels.
-
-        Parameters
-        ----------
-        compound : str
-
-        Returns
-        -------
-        isotopomers : list(str)
-        """
+    def get_compound_isotopomers_with_n_labels(self, compound: str, n_labels: int) -> list[str]:
+        """Get all isotopomers of a compound, that have excactly n labels."""
         label_positions = self.label_compounds[compound]["num_labels"]
         label_patterns = [
             ["1" if i in positions else "0" for i in range(label_positions)]
@@ -268,14 +221,11 @@ class LabelModel(Model):
         ]
         return [f"{compound}__{''.join(i)}" for i in label_patterns]
 
-    def get_compound_isotopomer_with_label_position(self, base_compound, label_position):
+    def get_compound_isotopomer_with_label_position(
+        self, base_compound: str, label_position: Union[int, list[int]]
+    ) -> list[str]:
         """Get compound isotopomer with a given label position.
 
-        Parameters
-        ----------
-        base_compound : str
-        label_position : Union(int, iterable(int))
-
         Examples
         --------
         >>> add_label_compounds({"x": 2})
@@ -286,39 +236,35 @@ class LabelModel(Model):
         if isinstance(label_position, int):
             label_position = [label_position]
         return f"{base_compound}__" + "".join(
-            "1" if idx in label_position else "0" for idx in range(self.label_compounds[base_compound]["num_labels"])
+            "1" if idx in label_position else "0"
+            for idx in range(self.label_compounds[base_compound]["num_labels"])
         )
 
     @staticmethod
-    def _split_label_string(label, *, labels_per_compound):
+    def _split_label_string(label: str, *, labels_per_compound: list[str]) -> list[str]:
         """Split label string according to labels given in label list.
 
         The labels in the label list correspond to the number of
         label positions in the compound.
 
-        Parameters
-        ----------
-        labels : str
-        labels_per_compound : list(int)
-
         Examples
         --------
-        >>>_split_label_string(label="01", labels_per_compound=[2])
+        >>> _split_label_string(label="01", labels_per_compound=[2])
         ["01"]
 
-        >>>_split_label_string(label="01", labels_per_compound=[1, 1])
+        >>> _split_label_string(label="01", labels_per_compound=[1, 1])
         ["0", "1"]
 
-        >>>_split_label_string(label="0011", labels_per_compound=[4])
+        >>> _split_label_string(label="0011", labels_per_compound=[4])
         ["0011"]
 
-        >>>_split_label_string(label="0011", labels_per_compound=[3, 1])
+        >>> _split_label_string(label="0011", labels_per_compound=[3, 1])
         ["001", "1"]
 
-        >>>_split_label_string(label="0011", labels_per_compound=[2, 2])
+        >>> _split_label_string(label="0011", labels_per_compound=[2, 2])
         ["00", "11"]
 
-        >>>_split_label_string(label="0011", labels_per_compound=[1, 3])
+        >>> _split_label_string(label="0011", labels_per_compound=[1, 3])
         ["0", "011"]
         """
         split_labels = []
@@ -329,18 +275,12 @@ class LabelModel(Model):
         return split_labels
 
     @staticmethod
-    def _map_substrates_to_products(*, rate_suffix, labelmap):
-        """Map the rate_suffix to products using the labelmap.
-
-        Parameters
-        ----------
-        rate_suffix : str
-        labelmap : iterable(int)
-        """
+    def _map_substrates_to_products(*, rate_suffix: str, labelmap: list[int]) -> list[str]:
+        """Map the rate_suffix to products using the labelmap."""
         return "".join([rate_suffix[i] for i in labelmap])
 
     @staticmethod
-    def _unpack_stoichiometries(*, stoichiometries):
+    def _unpack_stoichiometries(*, stoichiometries: dict[str, int]) -> tuple[list[str]]:
         """Split stoichiometries into substrates and products.
 
         Parameters
@@ -361,20 +301,11 @@ class LabelModel(Model):
                 products.extend([k] * v)
         return substrates, products
 
-    def _get_labels_per_compound(self, *, compounds):
+    def _get_labels_per_compound(self, *, compounds: list[str]) -> list[int]:
         """Get labels per compound.
 
         This is used for _split_label string. Adds 0 for non-label compounds,
-        to show that they get no label.
-
-        Parameters
-        ----------
-        compounds : list(str)
-
-        Returns
-        -------
-        label_per_compound : list(int)
-        """
+        to show that they get no label."""
         labels_per_compound = []
         for compound in compounds:
             try:
@@ -386,18 +317,8 @@ class LabelModel(Model):
         return labels_per_compound
 
     @staticmethod
-    def _repack_stoichiometries(*, new_substrates, new_products):
-        """Pack substrates and products into stoichiometric dict.
-
-        Parameters
-        ----------
-        new_substrates : iterable(str)
-        new_products : iterable(str)
-
-        Returns
-        -------
-        stoichiometries : dict(str: int)
-        """
+    def _repack_stoichiometries(*, new_substrates: list[str], new_products: list[str]) -> dict[str, int]:
+        """Pack substrates and products into stoichiometric dict."""
         new_stoichiometries = _defaultdict(int)
         for arg in new_substrates:
             new_stoichiometries[arg] -= 1
@@ -406,18 +327,8 @@ class LabelModel(Model):
         return dict(new_stoichiometries)
 
     @staticmethod
-    def _assign_compound_labels(*, base_compounds, label_suffixes):
-        """Assign the correct suffixes.
-
-        Parameters
-        ----------
-        base_compounds : iterable(str)
-        label_suffixes : iterable(str)
-
-        Returns
-        -------
-        new_compounds : list(str)
-        """
+    def _assign_compound_labels(*, base_compounds: list[str], label_suffixes: list[str]) -> list[str]:
+        """Assign the correct suffixes."""
         new_compounds = []
         for i, compound in enumerate(base_compounds):
             if label_suffixes[i] != "":
@@ -428,12 +339,13 @@ class LabelModel(Model):
 
     def add_algebraic_module(
         self,
-        module_name,
-        function,
-        compounds=None,
-        derived_compounds=None,
-        modifiers=None,
-        parameters=None,
+        module_name: str,
+        function: callable,
+        compounds: list[str] = None,
+        derived_compounds: list[str] = None,
+        modifiers: list[str] = None,
+        parameters: list[str] = None,
+        args: list[str] = None,
     ):
         """Add an algebraic module to the model.
 
@@ -446,19 +358,19 @@ class LabelModel(Model):
 
         Parameters
         ----------
-        module_name : str
+        module_name
             Name of the module
-        function : callable
+        function
             Python method of the algebraic module
-        compounds : iterable(str)
+        compounds
             Names of compounds used for module
-        derived_compounds : iterable(str)
+        derived_compounds
             Names of compounds which are calculated by the module
-        modifiers : iterable(str)
+        modifiers
             Names of compounds which act as modifiers on the module
-        parameters : iterable(str)
+        parameters
             Names of the parameters which are passed to the function
-        meta_info : dict, optional
+        meta_info
             Meta info of the algebraic module. Allowed keys are
             {common_name, notes, database_links}
 
@@ -469,18 +381,18 @@ class LabelModel(Model):
 
         Examples
         --------
-        def rapid_equilibrium(substrate, k_eq):
-            x = substrate / (1 + k_eq)
-            y = substrate * k_eq / (1 + k_eq)
-            return x, y
+        >>> def rapid_equilibrium(substrate, k_eq):
+        >>>    x = substrate / (1 + k_eq)
+        >>>    y = substrate * k_eq / (1 + k_eq)
+        >>>    return x, y
 
-        add_algebraic_module(
-            module_name="fast_eq",
-            function=rapid_equilibrium,
-            compounds=["A"],
-            derived_compounds=["X", "Y"],
-            parameters=["K"],
-        )
+        >>> add_algebraic_module(
+        >>>     module_name="fast_eq",
+        >>>     function=rapid_equilibrium,
+        >>>     compounds=["A"],
+        >>>     derived_compounds=["X", "Y"],
+        >>>     parameters=["K"],
+        >>> )
         """
         if compounds is not None:
             compounds = [i + "__total" if i in self.label_compounds else i for i in compounds]
@@ -493,9 +405,17 @@ class LabelModel(Model):
             derived_compounds=derived_compounds,
             modifiers=modifiers,
             parameters=parameters,
+            args=args,
         )
 
-    def _get_external_labels(self, *, rate_name, total_product_labels, total_substrate_labels, external_labels):
+    def _get_external_labels(
+        self,
+        *,
+        rate_name: str,
+        total_product_labels: int,
+        total_substrate_labels: int,
+        external_labels: list[str],
+    ) -> str:
         n_external_labels = total_product_labels - total_substrate_labels
         if n_external_labels > 0:
             if not external_labels:
@@ -513,17 +433,37 @@ class LabelModel(Model):
     def _add_base_reaction(
         self,
         *,
-        rate_name,
-        function,
-        stoichiometry,
-        labelmap,
-        external_labels,
-        modifiers,
-        parameters,
-        reversible,
-        variants,
+        rate_name: str,
+        function: callable,
+        stoichiometry: dict[str, float],
+        base_substrates: list[str],
+        base_products: list[str],
+        labelmap: list[int],
+        external_labels: str,
+        modifiers: list[str],
+        parameters: list[str],
+        reversible: bool,
+        # dynamic_variables: list[str] = None,
+        # args: list[str] = None,
+        variants: list[str],
         **meta_info,
     ):
+        # The check isn't as easy.
+        # Don't compare to the compounds, probably against base compounds, non-isotopomers and derived compounds?
+        # if parameters is None:
+        #     parameters = []
+        # if modifiers is None:
+        #     modifiers = []
+        # if reversible:
+        #     dynamic_variables = base_substrates + base_products + modifiers
+        # else:
+        #     dynamic_variables = base_substrates + modifiers
+        # self._check_for_existence(
+        #     name=rate_name, check_against=self.get_all_compounds(), candidates=dynamic_variables
+        # )
+        # self._check_for_existence(
+        #     name=rate_name, check_against=self.get_parameter_names(), candidates=parameters
+        # )
         self.base_reactions[rate_name] = {
             "function": function,
             "stoichiometry": stoichiometry,
@@ -539,32 +479,42 @@ class LabelModel(Model):
     def _create_isotopomer_reactions(
         self,
         *,
-        rate_name,
-        function,
-        labelmap,
-        modifiers,
-        parameters,
-        reversible,
-        external_labels,
-        total_substrate_labels,
-        base_substrates,
-        base_products,
-        labels_per_substrate,
-        labels_per_product,
+        rate_name: str,
+        function: callable,
+        labelmap: list[int],
+        modifiers: list[str],
+        parameters: list[str],
+        reversible: bool,
+        # dynamic_variables: list[str],
+        # args: list[str],
+        external_labels: str,
+        total_substrate_labels: int,
+        base_substrates: list[str],
+        base_products: list[str],
+        labels_per_substrate: list[str],
+        labels_per_product: list[str],
     ):
         variants = []
         for rate_suffix in ("".join(i) for i in it.product(("0", "1"), repeat=total_substrate_labels)):
             rate_suffix += external_labels
             # This is the magic
             product_suffix = self._map_substrates_to_products(rate_suffix=rate_suffix, labelmap=labelmap)
-            product_labels = self._split_label_string(label=product_suffix, labels_per_compound=labels_per_product)
-            substrate_labels = self._split_label_string(label=rate_suffix, labels_per_compound=labels_per_substrate)
+            product_labels = self._split_label_string(
+                label=product_suffix, labels_per_compound=labels_per_product
+            )
+            substrate_labels = self._split_label_string(
+                label=rate_suffix, labels_per_compound=labels_per_substrate
+            )
 
             new_substrates = self._assign_compound_labels(
                 base_compounds=base_substrates, label_suffixes=substrate_labels
             )
-            new_products = self._assign_compound_labels(base_compounds=base_products, label_suffixes=product_labels)
-            new_stoichiometry = self._repack_stoichiometries(new_substrates=new_substrates, new_products=new_products)
+            new_products = self._assign_compound_labels(
+                base_compounds=base_products, label_suffixes=product_labels
+            )
+            new_stoichiometry = self._repack_stoichiometries(
+                new_substrates=new_substrates, new_products=new_products
+            )
             new_rate_name = rate_name + "__" + rate_suffix
             self.add_reaction(
                 rate_name=new_rate_name,
@@ -573,6 +523,9 @@ class LabelModel(Model):
                 modifiers=modifiers,
                 parameters=parameters,
                 reversible=reversible,
+                # dynamic_variables=dynamic_variables,
+                # args=args,
+                check_consistency=False,
             )
             del self.meta_info["rates"][new_rate_name]
             variants.append(new_rate_name)
@@ -580,37 +533,39 @@ class LabelModel(Model):
 
     def add_labelmap_reaction(
         self,
-        rate_name,
-        function,
-        stoichiometry,
-        labelmap,
-        external_labels=None,
-        modifiers=None,
-        parameters=None,
-        reversible=False,
+        rate_name: str,
+        function: callable,
+        stoichiometry: dict[str, float],
+        labelmap: list[int],
+        external_labels: Union[int, list[int]] = None,
+        modifiers: list[str] = None,
+        parameters: list[str] = None,
+        reversible: bool = False,
+        # dynamic_variables: list[str] = None,
+        # args: list[str] = None,
         **meta_info,
     ):
         """Add a labelmap reaction.
 
         Parameters
         ----------
-        rate_name : str
+        rate_name
             Name of the rate function
-        function : callable
+        function
             Python method calculating the rate equation
-        stoichiometry : dict(str: int)
+        stoichiometry
             stoichiometry of the reaction
-        labelmap : iterable(int)
+        labelmap
             Mapping of the product label positions to the substrates
-        external_labels: iterable(int)
+        external_labels
             Positions in which external labels are supposed to be inserted
-        modifiers: iterable(str)
+        modifiers
             Names of the modifiers. E.g time.
-        parameters: iterable(str)
+        parameters
             Names of the parameters
-        reversible: bool
+        reversible
             Whether the reaction is reversible.
-        meta_info : dict, optional
+        meta_info
             Meta info of the rate. Allowed keys are
             {common_name, gibbs0, ec, database_links, notes, sbml_function}
 
@@ -659,6 +614,8 @@ class LabelModel(Model):
             modifiers=modifiers,
             parameters=parameters,
             reversible=reversible,
+            # dynamic_variables=dynamic_variables,
+            # args=args,
             external_labels=external_labels,
             total_substrate_labels=total_substrate_labels,
             base_substrates=base_substrates,
@@ -670,50 +627,52 @@ class LabelModel(Model):
         self._add_base_reaction(
             rate_name=rate_name,
             function=function,
+            base_substrates=base_substrates,
+            base_products=base_products,
             stoichiometry=stoichiometry,
             labelmap=labelmap,
             external_labels=external_labels,
             modifiers=modifiers,
             parameters=parameters,
             reversible=reversible,
+            # dynamic_variables=dynamic_variables,
+            # args=args,
             variants=variants,
             **meta_info,
         )
 
     def update_labelmap_reaction(
         self,
-        rate_name,
-        function=None,
-        stoichiometry=None,
-        labelmap=None,
-        modifiers=None,
-        parameters=None,
-        reversible=None,
-        **meta_info,
+        rate_name: str,
+        function: callable = None,
+        stoichiometry: dict[str, int] = None,
+        labelmap: list[int] = None,
+        modifiers: list[str] = None,
+        parameters: list[str] = None,
+        reversible: bool = None,
+        # dynamic_variables: list[str] = None,
+        # args: list[str] = None,
     ):
         """Update an existing labelmap reaction.
 
         Parameters
         ----------
-        rate_name : str
+        rate_name
             Name of the rate function
-        function : callable, optional
+        function
             Python method calculating the rate equation
-        stoichiometry : dict(str: int), optional
+        stoichiometry
             stoichiometry of the reaction
-        labelmap : iterable(int), optional
+        labelmap
             Mapping of the product label positions to the substrates
-        external_labels: iterable(int), optional
+        external_labels
             Positions in which external labels are supposed to be inserted
-        modifiers: iterable(str), optional
+        modifiers
             Names of the modifiers. E.g time.
-        parameters: iterable(str), optional
+        parameters
             Names of the parameters
-        reversible: bool, optional
+        reversible
             Whether the reaction is reversible.
-        meta_info : dict, optional
-            Meta info of the rate. Allowed keys are
-            {common_name, gibbs0, ec, database_links, notes, sbml_function}
         """
         if function is None:
             function = self.base_reactions[rate_name]["function"]
@@ -727,8 +686,6 @@ class LabelModel(Model):
             parameters = self.base_reactions[rate_name]["parameters"]
         if reversible is None:
             reversible = self.base_reactions[rate_name]["reversible"]
-        meta = self.meta_info["rates"][rate_name].__dict__
-        meta.update(meta_info)
 
         self.remove_labelmap_reaction(rate_name=rate_name)
         self.add_labelmap_reaction(
@@ -739,9 +696,11 @@ class LabelModel(Model):
             modifiers=modifiers,
             parameters=parameters,
             reversible=reversible,
+            # dynamic_variables=dynamic_variables,
+            # args=args,
         )
 
-    def remove_labelmap_reaction(self, rate_name):
+    def remove_labelmap_reaction(self, rate_name: str):
         """Remove all variants of a base reaction.
 
         Parameters
@@ -755,7 +714,7 @@ class LabelModel(Model):
             if rate.startswith(rate_name):
                 self.remove_reaction(rate_name=rate)
 
-    def remove_labelmap_reactions(self, rate_names):
+    def remove_labelmap_reactions(self, rate_names: list[str]):
         """Remove all variants of a multiple labelmap reactions.
 
         Parameters
@@ -769,23 +728,16 @@ class LabelModel(Model):
         for rate_name in rate_names:
             self.remove_labelmap_reaction(rate_name=rate_name)
 
-    # Simulation functions
-    def generate_y0(self, base_y0, label_positions=None):
+    def generate_y0(
+        self, base_y0: dict[str, float], label_positions: dict[str, Union[int, list[int]]] = None
+    ) -> dict[str, float]:
         """Generate y0 for all isotopomers given a base y0.
 
-        Parameters
-        ----------
-        base_y0 : dict(str: num)
-        label_positions: dict(str: num)
-
-        Returns
-        -------
-        y0 : dict(str: num)
-
         Examples
         --------
         >>> base_y0 = {"GAP": 1, "DHAP": 0, "FBP": 0}
         >>> generate_y0(base_y0=base_y0, label_positions={"GAP": 0})
+        >>> generate_y0(base_y0=base_y0, label_positions={"GAP": [0, 1, 2]})
         """
         if label_positions is None:
             label_positions = {}
@@ -810,7 +762,12 @@ class LabelModel(Model):
                 y0[base_compound + suffix] = concentration
         return y0
 
-    def get_total_fluxes(self, rate_base_name, y, t=0):
+    def get_total_fluxes(
+        self,
+        rate_base_name: str,
+        y: Union[dict[str, float], dict[str, list[float]]],
+        t: Union[float, list[float]] = 0,
+    ):
         """Get total fluxes of a base rate.
 
         Parameters
@@ -826,19 +783,12 @@ class LabelModel(Model):
         rates = [i for i in self.rates if i.startswith(rate_base_name + "__")]
         return self.get_fluxes_df(y=y, t=t)[rates].sum(axis=1).values
 
-    def _create_label_scope_seed(self, *, initial_labels):
-        """Create initial label scope seed.
-
-        Parameters
-        ----------
-        initial_labels : iterable(str)
-
-        Returns
-        -------
-        labelled_compounds : set(str)
-        """
+    def _create_label_scope_seed(self, *, initial_labels: dict[str, int]) -> dict[str, int]:
+        """Create initial label scope seed."""
         initial_labels = [
-            self.get_compound_isotopomer_with_label_position(base_compound=base_compound, label_position=label_position)
+            self.get_compound_isotopomer_with_label_position(
+                base_compound=base_compound, label_position=label_position
+            )
             for base_compound, label_position in initial_labels.items()
         ]
 
@@ -857,7 +807,10 @@ class LabelModel(Model):
             labelled_compounds[i] = 1
         return labelled_compounds
 
-    def get_label_scope(self, initial_labels):
+    def get_label_scope(
+        self,
+        initial_labels: Union[dict[str, int], dict[str, list[int]]],
+    ) -> dict[int, set[str]]:
         """Return all label positions that can be reached step by step.
 
         Parameters
@@ -923,13 +876,13 @@ class LabelModel(Model):
     # Model conversion functions
     ##########################################################################
 
-    def _map_label_compound_to_compound(self, *, compound):
+    def _map_label_compound_to_compound(self, *, compound: str) -> str:
         if compound in self.nonlabel_compounds:
             return compound
         else:
             return compound.rsplit("__")[0]
 
-    def to_model(self):
+    def to_model(self) -> Model:
         """Convert LabelModel to Model."""
         m = Model()
         m.add_parameters(self.parameters)
@@ -939,19 +892,26 @@ class LabelModel(Model):
         for module_name, module in self.algebraic_modules.items():
             if not module_name.endswith("__total"):
                 module = module.copy()
-                module["compounds"] = [self._map_label_compound_to_compound(compound=i) for i in module["compounds"]]
+                module["compounds"] = [
+                    self._map_label_compound_to_compound(compound=i) for i in module["compounds"]
+                ]
                 module["derived_compounds"] = [
                     self._map_label_compound_to_compound(compound=i) for i in module["derived_compounds"]
                 ]
-                module["modifiers"] = [self._map_label_compound_to_compound(compound=i) for i in module["modifiers"]]
+                module["modifiers"] = [
+                    self._map_label_compound_to_compound(compound=i) for i in module["modifiers"]
+                ]
                 m.add_algebraic_module(module_name=module_name, **module)
 
         for rate_name, reaction in self.base_reactions.items():
             reaction = reaction.copy()
             reaction["stoichiometry"] = {
-                self._map_label_compound_to_compound(compound=k): v for k, v in reaction["stoichiometry"].items()
+                self._map_label_compound_to_compound(compound=k): v
+                for k, v in reaction["stoichiometry"].items()
             }
-            reaction["modifiers"] = [self._map_label_compound_to_compound(compound=i) for i in reaction["modifiers"]]
+            reaction["modifiers"] = [
+                self._map_label_compound_to_compound(compound=i) for i in reaction["modifiers"]
+            ]
             del reaction["labelmap"]
             del reaction["external_labels"]
             del reaction["variants"]
@@ -962,7 +922,16 @@ class LabelModel(Model):
     # SBML functions
     ##########################################################################
 
-    def _add_sbml_compound(self, *, sbml_model, compound_id, common_name, compartment, charge, formula):
+    def _add_sbml_compound(
+        self,
+        *,
+        sbml_model: libsbml.Model,
+        compound_id: str,
+        common_name: str,
+        compartment: str,
+        charge: int,
+        formula: str,
+    ):
         cpd = sbml_model.createSpecies()
         cpd.setId(convert_id_to_sbml(id_=compound_id, prefix="CPD"))
         if common_name is not None:
@@ -978,7 +947,7 @@ class LabelModel(Model):
         if formula is not None:
             cpd_fbc.setChemicalFormula(formula)
 
-    def _create_sbml_compounds(self, *, sbml_model):
+    def _create_sbml_compounds(self, *, sbml_model: libsbml.Model):
         """Create the compounds for the sbml model.
 
         Parameters
@@ -1017,7 +986,7 @@ class LabelModel(Model):
                 formula=formula,
             )
 
-    def _create_sbml_reactions(self, *, sbml_model):
+    def _create_sbml_reactions(self, *, sbml_model: libsbml.Model):
         """Create the reactions for the sbml model.
 
         Parameters
