@@ -2,17 +2,16 @@
 
 Description of the module
 """
+from __future__ import annotations
 
-# Standard Library
 import subprocess
 import warnings
+from typing import Union, Iterable
 
-# Third party
 import libsbml
 import numpy as np
 import pandas as pd
 
-# Local code
 from ...core import (
     AlgebraicMixin,
     BaseModel,
@@ -36,13 +35,13 @@ class Model(
 
     def __init__(
         self,
-        parameters=None,
-        compounds=None,
-        algebraic_modules=None,
-        rate_stoichiometries=None,
-        rates=None,
-        functions=None,
-        meta_info=None,
+        parameters: dict = None,
+        compounds: dict = None,
+        algebraic_modules: dict = None,
+        rate_stoichiometries: dict = None,
+        rates: dict = None,
+        functions: dict = None,
+        meta_info: dict = None,
     ):
         BaseModel.__init__(self, meta_info=meta_info)
         CompoundMixin.__init__(self, compounds=compounds)
@@ -65,34 +64,172 @@ class Model(
             + f"\n    {len(self.get_stoichiometries())} Reactions"
         )
 
+    def _element_difference(self, other: Model, attribute: str):
+        self_collection = getattr(self, attribute)
+        other_collection = getattr(other, attribute)
+        difference = sorted(set(other_collection).difference(self_collection))
+        if not difference:
+            return None
+        if attribute == "compounds":
+            return difference
+        return {k: other_collection[k] for k in difference}
+
+    def _element_intersection(self, other: Model, attribute: str):
+        self_collection = getattr(self, attribute)
+        other_collection = getattr(other, attribute)
+        intersection: list[str] = sorted(set(self_collection).intersection(other_collection))
+        if not intersection:
+            return None
+        if attribute == "compounds":
+            return intersection
+        return {k: other_collection[k] for k in intersection}
+
+    def __add__(self, other: Model) -> Model:
+        m = self.copy()
+        m.add(**{"compounds": self._element_difference(other, "compounds")})
+
+        for name, module in other.algebraic_modules.items():
+            if name in self.algebraic_modules:
+                m.update_algebraic_module(name, **module)
+            else:
+                m.add_algebraic_module(name, **module)
+
+        for attribute in [
+            "parameters",
+            "rates",
+            "stoichiometries",
+            "functions",
+        ]:
+            m.add(**{attribute: self._element_difference(other, attribute)})
+            m.update(**{attribute: self._element_intersection(other, attribute)})
+        return m
+
+    def __iadd__(self, other: Model) -> Model:
+        self.add(**{"compounds": self._element_difference(other, "compounds")})
+        for name, module in other.algebraic_modules.items():
+            if name in self.algebraic_modules:
+                self.update_algebraic_module(name, **module)
+            else:
+                self.add_algebraic_module(name, **module)
+
+        for attribute in [
+            "parameters",
+            "rates",
+            "stoichiometries",
+            "functions",
+        ]:
+            self.add(**{attribute: self._element_difference(other, attribute)})
+            self.update(**{attribute: self._element_intersection(other, attribute)})
+        return self
+
+    def __sub__(self, other: Model) -> Model:
+        m = self.copy()
+        for attribute in [
+            "compounds",
+            "parameters",
+            "algebraic_modules",
+            "rates",
+            "stoichiometries",
+            "functions",
+        ]:
+            m.remove(**{attribute: self._element_intersection(other, attribute)})
+        return m
+
+    def __isub__(self, other: Model) -> Model:
+        for attribute in [
+            "compounds",
+            "parameters",
+            "algebraic_modules",
+            "rates",
+            "stoichiometries",
+            "functions",
+        ]:
+            self.remove(**{attribute: self._element_intersection(other, attribute)})
+        return self
+
+    def add(
+        self,
+        compounds: list = None,
+        parameters: dict[str, float] = None,
+        algebraic_modules: dict[str, dict] = None,
+        rates: dict[str, dict] = None,
+        stoichiometries: dict[str, dict] = None,
+        functions: dict[str, callable] = None,
+    ):
+        if compounds is not None:
+            self.add_compounds(compounds)
+        if parameters is not None:
+            self.add_parameters(parameters)
+        if algebraic_modules is not None:
+            self.add_algebraic_modules(algebraic_modules)
+        if rates is not None:
+            self.add_rates(rates)
+        if stoichiometries is not None:
+            self.add_stoichiometries(stoichiometries)
+        if functions is not None:
+            self.add_functions(functions)
+
+    def update(
+        self,
+        parameters: dict[str, float] = None,
+        algebraic_modules: dict[str, dict] = None,
+        rates: dict[str, dict] = None,
+        stoichiometries: dict[str, dict] = None,
+        functions: dict[str, callable] = None,
+    ):
+        if parameters is not None:
+            self.update_parameters(parameters)
+        if algebraic_modules is not None:
+            self.update_algebraic_modules(algebraic_modules)
+        if rates is not None:
+            self.update_rates(rates)
+        if stoichiometries is not None:
+            self.update_stoichiometries(stoichiometries)
+        if functions is not None:
+            self.update_functions(functions)
+
+    def remove(
+        self,
+        compounds: list[str] = None,
+        parameters: list[str] = None,
+        algebraic_modules: list[str] = None,
+        rates: list[str] = None,
+        stoichiometries: list[str] = None,
+        functions: list[str] = None,
+    ):
+        if compounds is not None:
+            self.remove_compounds(compounds)
+        if parameters is not None:
+            self.remove_parameters(parameters)
+        if algebraic_modules is not None:
+            self.remove_algebraic_modules(algebraic_modules)
+        if rates is not None:
+            self.remove_rates(rates)
+        if stoichiometries is not None:
+            self.remove_rate_stoichiometries(stoichiometries)
+        if functions is not None:
+            self.remove_functions(functions)
+
     ##########################################################################
     # Reactions
     ##########################################################################
 
     def add_reaction(
         self,
-        rate_name,
-        function,
-        stoichiometry,
-        modifiers=None,
-        dynamic_variables=None,
-        parameters=None,
-        reversible=False,
+        rate_name: str,
+        function: callable,
+        stoichiometry: dict[str, int],
+        modifiers: list[str] = None,
+        parameters: list[str] = None,
+        dynamic_variables: list[str] = None,
+        args: list[str] = None,
+        reversible: bool = False,
+        check_consistency: bool = True,
         **meta_info,
     ):
         """Add a reaction to the model.
 
         Shortcut for add_rate and add stoichiometry functions.
-        Additional variable ["time"] can be passed for time-dependent functions.
-
-        Parameters
-        ----------
-        rate_name : str
-        function : callable
-        stoichiometry : dict
-        modifiers: iterable(str)
-        parameters: iterable(str)
-        reversible: bool
 
         See Also
         --------
@@ -102,18 +239,19 @@ class Model(
         Examples
         --------
         >>> add_reaction(
-                rate_name="v1",
-                function=mass_action,
-                stoichiometry={"X": -1, "Y": 1},
-                parameters=["k2"],
-            )
+        >>>     rate_name="v1",
+        >>>     function=mass_action,
+        >>>     stoichiometry={"X": -1, "Y": 1},
+        >>>     parameters=["k2"],
+        >>> )
+
         >>> add_reaction(
-                rate_name="v1",
-                function=reversible_mass_action,
-                stoichiometry={"X": -1, "Y": 1},
-                parameters=["k1_fwd", "k1_bwd"],
-                reversible=True,
-            )
+        >>>     rate_name="v1",
+        >>>     function=reversible_mass_action,
+        >>>     stoichiometry={"X": -1, "Y": 1},
+        >>>     parameters=["k1_fwd", "k1_bwd"],
+        >>>     reversible=True,
+        >>> )
         """
         substrates = [k for k, v in stoichiometry.items() if v < 0]
         products = [k for k, v in stoichiometry.items() if v > 0]
@@ -127,20 +265,23 @@ class Model(
             modifiers=modifiers,
             parameters=parameters,
             reversible=reversible,
+            args=args,
+            check_consistency=check_consistency,
             **meta_info,
         )
         self.add_stoichiometry(rate_name=rate_name, stoichiometry=stoichiometry)
 
     def update_reaction(
         self,
-        rate_name,
-        function=None,
-        stoichiometry=None,
-        modifiers=None,
-        dynamic_variables=None,
-        parameters=None,
-        reversible=None,
-        **meta_info,
+        rate_name: str,
+        function: callable = None,
+        stoichiometry: dict[str, float] = None,
+        modifiers: list[str] = None,
+        parameters: list[str] = None,
+        dynamic_variables: list[str] = None,
+        args: list[str] = None,
+        reversible: bool = None,
+        check_consistency: bool = True,
     ):
         """Update an existing reaction.
 
@@ -159,39 +300,45 @@ class Model(
         update_rate
         update_stoichiometry
         """
-        if function is None:
-            function = self.rates[rate_name]["function"]
-        if stoichiometry is None:
-            stoichiometry = self.stoichiometries[rate_name]
-        if modifiers is None:
-            modifiers = self.rates[rate_name]["modifiers"]
-        if parameters is None:
-            parameters = self.rates[rate_name]["parameters"]
-        if reversible is None:
-            reversible = self.rates[rate_name]["reversible"]
-        meta = self.meta_info["rates"][rate_name].__dict__
-        meta.update(meta_info)
-        self.remove_reaction(rate_name=rate_name)
-        self.add_reaction(
+        if stoichiometry is not None:
+            substrates = [k for k, v in stoichiometry.items() if v < 0]
+            products = [k for k, v in stoichiometry.items() if v > 0]
+        else:
+            substrates = None
+            products = None
+        self.update_rate(
             rate_name=rate_name,
             function=function,
-            stoichiometry=stoichiometry,
+            substrates=substrates,
+            products=products,
             modifiers=modifiers,
-            dynamic_variables=dynamic_variables,
             parameters=parameters,
             reversible=reversible,
-            **meta,
+            dynamic_variables=dynamic_variables,
+            args=args,
+            check_consistency=check_consistency,
         )
+        if stoichiometry is not None:
+            self.update_stoichiometry(rate_name=rate_name, stoichiometry=stoichiometry)
 
-    def add_reaction_from_ratelaw(self, rate_name, ratelaw, **meta_info):
+    def update_reactions(self, reactions: dict):
+        for rate_name, reaction in reactions.items():
+            self.update_reaction(rate_name, **reaction)
+
+    def add_reaction_from_ratelaw(
+        self,
+        rate_name: str,
+        ratelaw: "modelbase.ode.utils.ratelaw.BaseRateLaw",
+        **meta_info,
+    ):
         """Add a reaction from a ratelaw.
 
         Parameters
         ----------
-        rate_name : str
+        rate_name
         ratelaw : modelbase.ode.utils.ratelaw.BaseRateLaw
             Ratelaw instance
-        meta_info : dict, optional
+        meta_info
 
         Examples
         --------
@@ -220,7 +367,7 @@ class Model(
         )
         self.add_stoichiometry(rate_name=rate_name, stoichiometry=ratelaw.stoichiometry)
 
-    def remove_reaction(self, rate_name):
+    def remove_reaction(self, rate_name: str):
         """Remove a reaction from the model.
 
         Parameters
@@ -230,7 +377,7 @@ class Model(
         self.remove_rate(rate_name=rate_name)
         self.remove_rate_stoichiometry(rate_name=rate_name)
 
-    def remove_reactions(self, rate_names):
+    def remove_reactions(self, rate_names: list[str]):
         """Remove multiple reactions from the model.
 
         Parameters
@@ -244,21 +391,15 @@ class Model(
     # Simulation functions
     ##########################################################################
 
-    def get_full_concentration_dict(self, y, t=0):
+    def get_full_concentration_dict(
+        self,
+        y: Union[dict[str, float], dict[str, list[float]], np.ndarray[float], list[float]],
+        t: float = 0.0,
+    ) -> dict[str, np.ndarray]:
         """Calculate the derived variables (at time(s) t).
 
-        Parameters
-        ----------
-        y : Union(dict(str: num), iterable(num))
-        t : Union(num, iterable(num)), optional
-
-        Returns
-        -------
-        y_full : dict
-            Dictionary of the concentrations of all compounds and derived compounds
-
-        Example
-        -------
+        Examples
+        --------
         >>> get_full_concentration_dict(y=[0, 0])
         >>> get_full_concentration_dict(y={"X": 0, "Y": 0})
         """
@@ -272,71 +413,43 @@ class Model(
             y = dict(zip(self.get_compounds(), (np.ones((len(t), 1)) * y).T))
         return {k: np.ones(len(t)) * v for k, v in self._get_fcd(t=t, y=y).items()}
 
-    def get_fluxes_dict(self, y, t=0):
-        """Calculate the fluxes at time point(s) t.
-
-        Parameters
-        ----------
-        y : Union(dict(str: num), iterable(num))
-        t : Union(num, iterable(num))
-
-        Returns
-        -------
-        fluxes : dict
-        """
+    def get_fluxes_dict(
+        self,
+        y: Union[dict[str, float], dict[str, list[float]], np.ndarray[float], list[float]],
+        t: float = 0.0,
+    ) -> dict[str, np.ndarray]:
+        """Calculate the fluxes at time point(s) t."""
         self._update_derived_parameters()
         fcd = self.get_full_concentration_dict(y=y, t=t)
         return {k: np.ones(len(fcd["time"])) * v for k, v in self._get_fluxes(fcd=fcd).items()}
 
-    def get_fluxes_array(self, y, t=0):
-        """Calculate the fluxes at time point(s) t.
-
-        Parameters
-        ----------
-        y : Union(dict(str: num), iterable(num))
-        t : Union(num, iterable(num))
-
-        Returns
-        -------
-        fluxes : numpy.array
-        """
+    def get_fluxes_array(
+        self,
+        y: Union[dict[str, float], dict[str, list[float]], np.ndarray[float], list[float]],
+        t: float = 0.0,
+    ) -> np.ndarray:
+        """Calculate the fluxes at time point(s) t."""
         return np.array([i for i in self.get_fluxes_dict(y=y, t=t).values()]).T
 
-    def get_fluxes_df(self, y, t=0):
-        """Calculate the fluxes at time point(s) t.
-
-        Parameters
-        ----------
-        y : Union(dict(str: num), iterable(num))
-        t : Union(num, iterable(num))
-
-        Returns
-        -------
-        fluxes : pandas.DataFrame
-        """
+    def get_fluxes_df(
+        self,
+        y: Union[dict[str, float], dict[str, list[float]], np.ndarray[float], list[float]],
+        t: float = 0.0,
+    ) -> pd.DataFrame:
+        """Calculate the fluxes at time point(s) t."""
         if isinstance(t, (int, float)):
             t = [t]
         return pd.DataFrame(data=self.get_fluxes_dict(y=y, t=t), index=t, columns=self.get_rate_names())
 
     # This can't get keyword-only arguments, as the integrators are calling it with
     # positional arguments
-    def _get_rhs(self, t, y):
+    def _get_rhs(self, t: float, y: list[float]) -> np.ndarray:
         """Calculate the right hand side of the ODE system.
 
         This is the more performant version of get_right_hand_side()
         and thus returns only an array instead of a dictionary.
 
         Watch out that this function swaps t and y!
-
-        Parameters
-        ----------
-        t : num
-        y : iterable(num)
-
-        Returns
-        -------
-        rhs : list
-            List of the right hand side
         """
         y = dict(zip(self.get_compounds(), np.array(y).reshape(-1, 1)))
         fcd = self._get_fcd(t=t, y=y)
@@ -348,14 +461,12 @@ class Model(
                 dxdt[k] += n * fluxes[flux]
         return np.array([dxdt[i] for i in compounds_local]).flatten()
 
-    def get_right_hand_side(self, y, t=0):
-        """Calculate the right hand side of the ODE system.
-
-        Parameters
-        ----------
-        y : Union(dict(str: num), iterable(num))
-        t : Union(num, iterable(num))
-        """
+    def get_right_hand_side(
+        self,
+        y: Union[dict[str, float], dict[str, list[float]], np.ndarray[float], list[float]],
+        t: float = 0.0,
+    ) -> dict[str, float]:
+        """Calculate the right hand side of the ODE system."""
         self._update_derived_parameters()
         y = self.get_full_concentration_dict(y=y, t=t)
         y = [y[i] for i in self.get_compounds()]
@@ -367,20 +478,11 @@ class Model(
     # Model conversion functions
     ##########################################################################
 
-    def to_labelmodel(self, labelcompounds, labelmaps):
+    def to_labelmodel(
+        self, labelcompounds: dict[str, int], labelmaps: dict[str, list[int]]
+    ) -> "modelbase.ode.LabelModel":
         """Create a LabelModel from this model.
 
-        Parameters
-        ----------
-        labelcompounds : dict
-            Mapping compound to the amount of labels they can carry
-        labelmaps : dict
-            Mapping reaction to labelmap
-
-        Returns
-        -------
-        LabelModel: modelbase.ode.LabelModel
-
         Examples
         --------
         >>> m = Model()
@@ -395,7 +497,7 @@ class Model(
         >>> labelmaps = {"TPI": [2, 1, 0]}
         >>> m.to_labelmodel(labelcompounds=labelcompounds, labelmaps=labelmaps)
         """
-        # Third party
+
         from modelbase.ode import LabelModel
 
         lm = LabelModel()
@@ -438,43 +540,34 @@ class Model(
                 )
         return lm
 
-    def to_linear_labelmodel(self, labelcompounds, labelmaps):
+    def to_linear_labelmodel(
+        self, labelcompounds: dict[str, int], labelmaps: dict[str, list[int]]
+    ) -> "modelbase.ode.LinearLabelModel":
         """Create a LinearLabelModel from this model.
 
         Watch out that for a linear label model reversible reactions have to be split
         into a forward and backward part.
 
-        Parameters
-        ----------
-        labelcompounds : dict
-            Mapping compound to the amount of labels they can carry
-        labelmaps : dict
-            Mapping reaction to labelmap
-
-        Returns
-        -------
-        LabelModel: modelbase.ode.LabelModel
-
         Examples
         --------
         >>> m = Model()
         >>> m.add_reaction(
-                rate_name="TPI_fwd",
-                function=_mass_action_1_1,
-                stoichiometry={"GAP": -1, "DHAP": 1},
-                parameters=["kf_TPI"],
-            )
+        >>>     rate_name="TPI_fwd",
+        >>>     function=_mass_action_1_1,
+        >>>     stoichiometry={"GAP": -1, "DHAP": 1},
+        >>>     parameters=["kf_TPI"],
+        >>> )
         >>> m.add_reaction(
-                rate_name="TPI_bwd",
-                function=mass_action_1_1,
-                stoichiometry={"DHAP": -1, "GAP": 1},
-                parameters=["kr_TPI"],
-            )
+        >>>     rate_name="TPI_bwd",
+        >>>     function=mass_action_1_1,
+        >>>     stoichiometry={"DHAP": -1, "GAP": 1},
+        >>>     parameters=["kr_TPI"],
+        >>> )
         >>> labelcompounds = {"GAP": 3, "DHAP": 3}
         >>> labelmaps = {"TPI_fwd": [2, 1, 0], 'TPI_bwd': [2, 1, 0]}
         >>> m.to_linear_labelmodel(labelcompounds=labelcompounds, labelmaps=labelmaps)
         """
-        # Third party
+
         from modelbase.ode import LinearLabelModel
 
         lm = LinearLabelModel()
@@ -504,20 +597,16 @@ class Model(
     # Source code functions
     ##########################################################################
 
-    def generate_model_source_code(self, linted=True, include_meta_info=False):
+    def generate_model_source_code(self, linted: bool = True, include_meta_info: bool = False) -> str:
         """Generate source code of the model.
 
         Parameters
         ----------
-        linted : bool
+        linted
             Whether the source code should be formatted via black.
             Usually it only makes sense to turn this off if there is an error somewhere.
-        include_meta_info : bool
+        include_meta_info
             Whether to include meta info of the model components
-
-        Returns
-        -------
-        model_source_code : str
         """
         parameters = self._generate_parameters_source_code(include_meta_info=include_meta_info)
         compounds = self._generate_compounds_source_code(include_meta_info=include_meta_info)
@@ -554,13 +643,8 @@ class Model(
     # SBML functions
     ##########################################################################
 
-    def _create_sbml_reactions(self, *, sbml_model):
-        """Create the reactions for the sbml model.
-
-        Parameters
-        ----------
-        sbml_model : libsbml.Model
-        """
+    def _create_sbml_reactions(self, *, sbml_model: libsbml.Model):
+        """Create the reactions for the sbml model."""
         for rate_id, stoichiometry in self.stoichiometries.items():
             rate = self.meta_info["rates"][rate_id]
             rxn = sbml_model.createReaction()
@@ -589,7 +673,7 @@ class Model(
                 kinetic_law = rxn.createKineticLaw()
                 kinetic_law.setMath(libsbml.parseL3Formula(function))
 
-    def _model_to_sbml(self):
+    def _model_to_sbml(self) -> libsbml.SBMLDocument:
         """Export model to sbml."""
         doc = self._create_sbml_document()
         sbml_model = self._create_sbml_model(doc=doc)
