@@ -1,15 +1,15 @@
 """Write me."""
+from __future__ import annotations
 
-# Standard Library
 import math
+from typing import Union, Any
 
-# Third party
 import matplotlib.cm as cm
+from matplotlib.collections import QuadMesh
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 
-# Local code
 from ...utils.plotting import get_norm as _get_norm
 from ...utils.plotting import heatmap_from_dataframe as _heatmap_from_dataframe
 from ..__init__ import Simulator
@@ -17,45 +17,29 @@ from ..__init__ import Simulator
 _DISPLACEMENT = 1e-4
 
 
-def _find_steady_state(*, model, y0):
-    """Simulate the system to steadt state.
-
-    Just a convenience mapper.
-
-    Parameters
-    ----------
-    model : modelbase.ode.model
-    y0 : Union(dict(str: num), iterable(num))
-
-    Returns
-    -------
-    t : numpy.array
-    y : numpy.array
-    """
+def _find_steady_state(
+    *,
+    model,
+    y0: dict[str, float],
+) -> tuple[np.ndarray, np.ndarray]:
+    """Simulate the system to steadt state."""
     s = Simulator(model=model)
     s.initialise(y0=y0, test_run=False)
     t, y = s.simulate_to_steady_state()
     return t, y
 
 
-def get_compound_elasticity(model, compound, y, t=0, normalized=True, displacement=_DISPLACEMENT):
+def get_compound_elasticity(
+    model,
+    compound: str,
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> np.ndarray:
     """Get sensitivity of all rates to a change of the concentration of a compound.
 
     Also called epsilon-elasticities. Not in steady state!
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    compound: str
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    elasticities: numpy.ndarray
     """
     y = model.get_full_concentration_dict(y=y, t=t)
     old_concentration = y[compound]
@@ -74,24 +58,17 @@ def get_compound_elasticity(model, compound, y, t=0, normalized=True, displaceme
     return np.atleast_1d(np.squeeze(elasticity_coef))
 
 
-def get_compound_elasticities_array(model, compounds, y, t=0, normalized=True, displacement=_DISPLACEMENT):
+def get_compound_elasticities_array(
+    model,
+    compounds: list[str],
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> np.ndarray:
     """Get sensitivity of all rates to a change of the concentration of multiple compounds.
 
     Also called epsilon-elasticities. Not in steady state!
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    compounds: iterable(str)
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    elasticities: numpy.ndarray
     """
     elasticities = np.full(shape=(len(compounds), len(model.get_rate_names())), fill_value=np.nan)
     for i, compound in enumerate(compounds):
@@ -106,24 +83,17 @@ def get_compound_elasticities_array(model, compounds, y, t=0, normalized=True, d
     return elasticities
 
 
-def get_compound_elasticities_df(model, compounds, y, t=0, normalized=True, displacement=_DISPLACEMENT):
+def get_compound_elasticities_df(
+    model,
+    compounds: list[str],
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> pd.DataFrame:
     """Get sensitivity of all rates to a change of the concentration of multiple compounds.
 
     Also called epsilon-elasticities. Not in steady state!
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    compounds: iterable(str)
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    elasticities: pandas.DataFrame
     """
     array = get_compound_elasticities_array(
         model=model,
@@ -136,24 +106,17 @@ def get_compound_elasticities_df(model, compounds, y, t=0, normalized=True, disp
     return pd.DataFrame(data=array, index=compounds, columns=model.get_rate_names())
 
 
-def get_parameter_elasticity(model, parameter, y, t=0, normalized=True, displacement=_DISPLACEMENT):
+def get_parameter_elasticity(
+    model,
+    parameter: str,
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> np.ndarray:
     """Get sensitivity of all rates to a change of a parameter value.
 
     Also called pi-elasticities. Not in steady state!
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    parameter: str
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    elasticities: numpy.array
     """
     model = model.copy()
     old_value = model.get_parameter(parameter_name=parameter)
@@ -169,24 +132,17 @@ def get_parameter_elasticity(model, parameter, y, t=0, normalized=True, displace
     return np.atleast_1d(np.squeeze(elasticity_coef))
 
 
-def get_parameter_elasticities_array(model, parameters, y, t=0, normalized=True, displacement=_DISPLACEMENT):
+def get_parameter_elasticities_array(
+    model,
+    parameters: list[str],
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> np.ndarray:
     """Get sensitivity of all rates to a change of multiple parameter values.
 
     Also called pi-elasticities. Not in steady state!
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    parameters: iterable(str)
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    elasticities: numpy.array
     """
     elasticities = np.full(shape=(len(parameters), len(model.get_rate_names())), fill_value=np.nan)
     for i, parameter in enumerate(parameters):
@@ -201,25 +157,17 @@ def get_parameter_elasticities_array(model, parameters, y, t=0, normalized=True,
     return elasticities
 
 
-def get_parameter_elasticities_df(model, parameters, y, t=0, normalized=True, displacement=_DISPLACEMENT):
+def get_parameter_elasticities_df(
+    model,
+    parameters,
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> pd.DataFrame:
     """Get sensitivity of all rates to a change of multiple parameter values.
 
-    Also called pi-elasticities. Not in steady state!
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    parameters: iterable(str)
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    elasticities: pandas.DataFrame
-    """
+    Also called pi-elasticities. Not in steady state!"""
     matrix = get_parameter_elasticities_array(
         model=model,
         parameters=parameters,
@@ -232,24 +180,14 @@ def get_parameter_elasticities_df(model, parameters, y, t=0, normalized=True, di
 
 
 def get_concentration_response_coefficient(
-    model, parameter, y, t=0, normalized=True, displacement=_DISPLACEMENT
-):
-    """Get response of the steady state concentrations to a change of the given parameter.
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    parameter : str
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    coefficients: numpy.array
-    """
+    model,
+    parameter: str,
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> np.ndarray:
+    """Get response of the steady state concentrations to a change of the given parameter."""
     model = model.copy()
     old_value = model.get_parameter(parameter_name=parameter)
     ss = []
@@ -269,24 +207,14 @@ def get_concentration_response_coefficient(
 
 
 def get_concentration_response_coefficients_array(
-    model, parameters, y, t=0, normalized=True, displacement=_DISPLACEMENT
-):
-    """Get response of the steady state concentrations to a change of the given parameters.
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    parameters: iterable(str)
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    coefficients: numpy.array
-    """
+    model,
+    parameters: list[str],
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> np.ndarray:
+    """Get response of the steady state concentrations to a change of the given parameters."""
     crs = np.full(shape=(len(parameters), len(model.get_compounds())), fill_value=np.nan)
     for i, parameter in enumerate(parameters):
         crs[i] = get_concentration_response_coefficient(
@@ -301,24 +229,14 @@ def get_concentration_response_coefficients_array(
 
 
 def get_concentration_response_coefficients_df(
-    model, parameters, y, t=0, normalized=True, displacement=_DISPLACEMENT
-):
-    """Get response of the steady state concentrations to a change of the given parameter m.
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    parameters: iterable(str)
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    coefficients: pandas.DataFrame
-    """
+    model,
+    parameters: list[str],
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> pd.DataFrame:
+    """Get response of the steady state concentrations to a change of the given parameters"""
     array = get_concentration_response_coefficients_array(
         model=model,
         parameters=parameters,
@@ -330,23 +248,15 @@ def get_concentration_response_coefficients_df(
     return pd.DataFrame(data=array, index=parameters, columns=model.get_compounds())
 
 
-def get_flux_response_coefficient(model, parameter, y, t=0, normalized=True, displacement=_DISPLACEMENT):
-    """Get response of the steady state fluxes to a change of the given parameter.
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    parameters: iterable(str)
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    coefficients: numpy.array
-    """
+def get_flux_response_coefficient(
+    model,
+    parameter,
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> np.ndarray:
+    """Get response of the steady state fluxes to a change of the given parameter."""
     model = model.copy()
     old_value = model.get_parameter(parameter_name=parameter)
     fluxes = []
@@ -367,24 +277,14 @@ def get_flux_response_coefficient(model, parameter, y, t=0, normalized=True, dis
 
 
 def get_flux_response_coefficients_array(
-    model, parameters, y, t=0, normalized=True, displacement=_DISPLACEMENT
-):
-    """Get response of the steady state fluxes to a change of the given parameters.
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    parameters: iterable(str)
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    coefficients: numpy.array
-    """
+    model,
+    parameters: list[str],
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> np.ndarray:
+    """Get response of the steady state fluxes to a change of the given parameters."""
     frc = np.full(shape=(len(parameters), len(model.get_rate_names())), fill_value=np.nan)
     for i, parameter in enumerate(parameters):
         frc[i] = get_flux_response_coefficient(
@@ -398,23 +298,15 @@ def get_flux_response_coefficients_array(
     return frc
 
 
-def get_flux_response_coefficients_df(model, parameters, y, t=0, normalized=True, displacement=_DISPLACEMENT):
-    """Get response of the steady state fluxes to a change of the given parameters.
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    parameters: iterable(str)
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    coefficients: numpy.array
-    """
+def get_flux_response_coefficients_df(
+    model,
+    parameters: list[str],
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+) -> pd.DataFrame:
+    """Get response of the steady state fluxes to a change of the given parameters."""
     array = get_flux_response_coefficients_array(
         model=model,
         parameters=parameters,
@@ -427,21 +319,21 @@ def get_flux_response_coefficients_df(model, parameters, y, t=0, normalized=True
 
 
 def plot_coefficient_heatmap(
-    df,
-    title,
-    cmap="RdBu_r",
-    rows=None,
-    columns=None,
-    vmax=None,
-    vmin=None,
-    norm=None,
-    annotate=True,
-    colorbar=True,
-    xlabel=None,
-    ylabel=None,
-    ax=None,
-    cax=None,
-):
+    df: pd.DataFrame,
+    title: str,
+    cmap: str = "RdBu_r",
+    rows: int = None,
+    columns: int = None,
+    vmax: float = None,
+    vmin: float = None,
+    norm: plt.Normalize = None,
+    annotate: bool = True,
+    colorbar: bool = True,
+    xlabel: str = None,
+    ylabel: str = None,
+    ax: plt.Axes = None,
+    cax: plt.Axes = None,
+) -> tuple[plt.Figure, plt.Axes, QuadMesh]:
     df = df.T.round(2)
     if rows is None:
         rows = df.index
@@ -465,7 +357,14 @@ def plot_coefficient_heatmap(
     return fig, ax, hm
 
 
-def plot_multiple(dfs, titles, cmap="RdBu_r", annotate=True, colorbar=True, figsize=None):
+def plot_multiple(
+    dfs: list[pd.DataFrame],
+    titles: list[str],
+    cmap: str = "RdBu_r",
+    annotate: bool = True,
+    colorbar: bool = True,
+    figsize: tuple[int, int] = None,
+) -> tuple[plt.Figure, np.ndarray[plt.Axes]]:
     vmin = min(i.values.min() for i in dfs)
     vmax = max(i.values.max() for i in dfs)
 
@@ -502,41 +401,24 @@ def plot_multiple(dfs, titles, cmap="RdBu_r", annotate=True, colorbar=True, figs
 
 def plot_concentration_response_coefficients(
     model,
-    parameters,
-    y,
-    t=0,
-    normalized=True,
-    displacement=_DISPLACEMENT,
-    cmap="RdBu_r",
-    rows=None,
-    columns=None,
-    vmax=None,
-    vmin=None,
-    annotate=True,
-    colorbar=True,
-    xlabel=None,
-    ylabel=None,
-    ax=None,
-    cax=None,
-):
-    """Plot response of the steady state concentration to a change of the given parameters.
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    parameters: iterable(str)
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    fig : matplotlib figure
-    ax : matplotlib axes
-    hm : matplotlib heatmap
-    """
+    parameters: list[str],
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+    cmap: str = "RdBu_r",
+    rows: int = None,
+    columns: int = None,
+    vmax: float = None,
+    vmin: float = None,
+    annotate: bool = True,
+    colorbar: bool = True,
+    xlabel: str = None,
+    ylabel: str = None,
+    ax: plt.Axes = None,
+    cax: plt.Axes = None,
+) -> tuple[plt.Figure, plt.Axes, QuadMesh]:
+    """Plot response of the steady state concentration to a change of the given parameters."""
     df = get_concentration_response_coefficients_df(
         model=model,
         parameters=parameters,
@@ -564,41 +446,24 @@ def plot_concentration_response_coefficients(
 
 def plot_flux_response_coefficients(
     model,
-    parameters,
-    y,
-    t=0,
-    normalized=True,
-    displacement=_DISPLACEMENT,
-    cmap="RdBu_r",
-    rows=None,
-    columns=None,
-    vmax=None,
-    vmin=None,
-    annotate=True,
-    colorbar=True,
-    xlabel=None,
-    ylabel=None,
-    ax=None,
-    cax=None,
-):
-    """Plot response of the steady state fluxes to a change of the given parameters.
-
-    Parameters
-    ----------
-    m: modelbase.ode.Model
-    parameters: iterable(str)
-    y: Union(dict(str: num), iterable(num))
-    t: Union(num, iterable(num))
-    normalized : bool
-    displacement : float
-        Percentage change of the value
-
-    Returns
-    -------
-    fig : matplotlib figure
-    ax : matplotlib axes
-    hm : matplotlib heatmap
-    """
+    parameters: list[str],
+    y: dict[str, float],
+    t: float = 0,
+    normalized: bool = True,
+    displacement: float = _DISPLACEMENT,
+    cmap: str = "RdBu_r",
+    rows: int = None,
+    columns: int = None,
+    vmax: float = None,
+    vmin: float = None,
+    annotate: bool = True,
+    colorbar: bool = True,
+    xlabel: str = None,
+    ylabel: str = None,
+    ax: plt.Axes = None,
+    cax: plt.Axes = None,
+) -> tuple[plt.Figure, plt.Axes, QuadMesh]:
+    """Plot response of the steady state fluxes to a change of the given parameters."""
     df = get_flux_response_coefficients_df(
         model=model,
         parameters=parameters,
