@@ -1,12 +1,13 @@
-# Standard Library
+from __future__ import annotations
+
 from abc import ABC, abstractmethod
 from collections import defaultdict
+from typing import Union
 
-# Local code
 from . import ratefunctions
 
 
-def _pack_stoichiometries(substrates, products):
+def _pack_stoichiometries(substrates: list[str], products: list[str]) -> dict[str, int]:
     new_stoichiometries = defaultdict(int)
     for arg in substrates:
         new_stoichiometries[arg] -= 1
@@ -17,53 +18,55 @@ def _pack_stoichiometries(substrates, products):
 
 class BaseRateLaw(ABC):
     def __init__(self):
-        self.substrates = []
-        self.products = []
-        self.modifiers = []
-        self.parameters = []
-        self.stoichiometry = {}
-        self.reversible = False
+        self.substrates: list[str] = []
+        self.products: list[str] = []
+        self.modifiers: list[str] = []
+        self.parameters: list[str] = []
+        self.stoichiometry: dict[str, int] = {}
+        self.reversible: bool = False
 
     @abstractmethod
-    def get_sbml_function_string(self):
+    def get_sbml_function_string(self) -> str:
         """Write me."""
 
     @abstractmethod
-    def get_rate_function(self):
+    def get_rate_function(self) -> callable:
         """Write me."""
 
 
 class Constant(BaseRateLaw):
-    def __init__(self, product, k):
+    def __init__(self, product: str, k: str):
         super().__init__()
         self.products = [product]
         self.parameters = [k]
         self.stoichiometry = {product: 1}
-
         self.k = k
 
-    def get_sbml_function_string(self):
+    def get_sbml_function_string(self) -> str:
         return f"{self.k}"
 
-    def get_rate_function(self):
+    def get_rate_function(self) -> callable:
         return ratefunctions.constant
 
 
 class MassAction(BaseRateLaw):
-    def __init__(self, substrates, products, k_fwd):
-
+    def __init__(
+        self,
+        substrates: Union[str, list[str]],
+        products: Union[str, list[str]],
+        k_fwd: str,
+    ):
         super().__init__()
         self.substrates = list(substrates) if not isinstance(substrates, str) else [substrates]
         self.products = list(products) if not isinstance(products, str) else [products]
         self.parameters = [k_fwd]
         self.stoichiometry = _pack_stoichiometries(substrates, products)
-
         self.k_fwd = k_fwd
 
-    def get_sbml_function_string(self):
+    def get_sbml_function_string(self) -> str:
         return f"{self.k_fwd} * {' * '.join(self.substrates)}"
 
-    def get_rate_function(self):
+    def get_rate_function(self) -> callable:
         try:
             return getattr(ratefunctions, f"mass_action_{len(self.substrates)}")
         except AttributeError:
@@ -71,7 +74,13 @@ class MassAction(BaseRateLaw):
 
 
 class ReversibleMassAction(BaseRateLaw):
-    def __init__(self, substrates, products, k_fwd, k_bwd):
+    def __init__(
+        self,
+        substrates: Union[str, list[str]],
+        products: Union[str, list[str]],
+        k_fwd: str,
+        k_bwd: str,
+    ):
         super().__init__()
         self.substrates = list(substrates) if not isinstance(substrates, str) else [substrates]
         self.products = list(products) if not isinstance(products, str) else [products]
@@ -82,10 +91,10 @@ class ReversibleMassAction(BaseRateLaw):
         self.k_fwd = k_fwd
         self.k_bwd = k_bwd
 
-    def get_sbml_function_string(self):
+    def get_sbml_function_string(self) -> str:
         return f"{self.k_fwd} * {' * '.join(self.substrates)} - {self.k_bwd} * {' * '.join(self.products)}"
 
-    def get_rate_function(self):
+    def get_rate_function(self) -> callable:
         try:
             return getattr(
                 ratefunctions,
@@ -99,8 +108,7 @@ class ReversibleMassAction(BaseRateLaw):
 
 
 class MichaelisMenten(BaseRateLaw):
-    def __init__(self, S, P, vmax, km):
-
+    def __init__(self, S: str, P: str, vmax: str, km: str):
         super().__init__()
         self.substrates = [S]
         self.products = [P]
@@ -111,15 +119,15 @@ class MichaelisMenten(BaseRateLaw):
         self.vmax = vmax
         self.km = km
 
-    def get_sbml_function_string(self):
+    def get_sbml_function_string(self) -> str:
         return f"{self.S} * {self.vmax} / ({self.S} + {self.km})"
 
-    def get_rate_function(self):
+    def get_rate_function(self) -> callable:
         return ratefunctions.michaelis_menten
 
 
 class ReversibleMichaelisMenten(BaseRateLaw):
-    def __init__(self, S, P, vmax_fwd, vmax_bwd, km_fwd, km_bwd):
+    def __init__(self, S: str, P: str, vmax_fwd: str, vmax_bwd: str, km_fwd: str, km_bwd: str):
         super().__init__()
         self.substrates = [S]
         self.products = [P]
@@ -134,18 +142,18 @@ class ReversibleMichaelisMenten(BaseRateLaw):
         self.km_fwd = km_fwd
         self.km_bwd = km_bwd
 
-    def get_sbml_function_string(self):
+    def get_sbml_function_string(self) -> str:
         return (
             f"({self.vmax_fwd} * {self.S} / {self.km_fwd} - {self.vmax_bwd} * {self.P} / {self.km_bwd})"
             + f" / (1 + {self.S} / {self.km_fwd} + {self.P} / {self.km_bwd})"
         )
 
-    def get_rate_function(self):
+    def get_rate_function(self) -> callable:
         return ratefunctions.reversible_michaelis_menten
 
 
 class ReversibleMichaelisMentenKeq(BaseRateLaw):
-    def __init__(self, S, P, vmax_fwd, km_fwd, km_bwd, k_eq):
+    def __init__(self, S: str, P: str, vmax_fwd: str, km_fwd: str, km_bwd: str, k_eq: str):
 
         super().__init__()
         self.substrates = [S]
@@ -162,11 +170,11 @@ class ReversibleMichaelisMentenKeq(BaseRateLaw):
         self.km_bwd = km_bwd
         self.k_eq = k_eq
 
-    def get_sbml_function_string(self):
+    def get_sbml_function_string(self) -> str:
         return (
             f"{self.vmax_fwd} / {self.km_fwd} * ({self.S} - {self.P} / {self.k_eq})"
             + f"/ (1 + {self.S} / {self.km_fwd} + {self.P} / {self.km_bwd})"
         )
 
-    def get_rate_function(self):
+    def get_rate_function(self) -> callable:
         return ratefunctions.reversible_michaelis_menten_keq
