@@ -1,4 +1,5 @@
-# Local code
+from __future__ import annotations
+
 from .unit_conversion import get_ast_types
 
 AST_TYPES = get_ast_types()
