@@ -1,7 +1,6 @@
-# Standard Library
+
 import unittest
 
-# Third party
 from modelbase.ode import LabelModel
 from modelbase.ode import ratefunctions as rf
 
@@ -1223,7 +1222,7 @@ class ModelConversionTests(unittest.TestCase):
         )
         m = label_model.to_model()
         self.assertEqual(m.compounds, ["a", "b", "x", "y"])
-        self.assertEqual(m.derived_compounds, ["c"])
+        self.assertEqual(m.get_derived_compounds(), ["c"])
 
         mod = m.algebraic_modules["mod1"]
         self.assertEqual(mod["compounds"], ["x", "a"])
