@@ -1,7 +1,6 @@
-# Standard Library
+
 import unittest
 
-# Third party
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
@@ -57,15 +56,18 @@ class LabelSimualtorTests(unittest.TestCase):
     def test_get_labeled_and_unlabeled(self):
         simulator = SIM.copy()
         np.testing.assert_array_almost_equal(
-            simulator.get_unlabeled_concentration(compound="x") + simulator.get_total_label_concentration(compound="x"),
+            simulator.get_unlabeled_concentration(compound="x")
+            + simulator.get_total_label_concentration(compound="x"),
             2.0,
         )
         np.testing.assert_array_almost_equal(
-            simulator.get_unlabeled_concentration(compound="y") + simulator.get_total_label_concentration(compound="y"),
+            simulator.get_unlabeled_concentration(compound="y")
+            + simulator.get_total_label_concentration(compound="y"),
             2.0,
         )
         np.testing.assert_array_almost_equal(
-            simulator.get_unlabeled_concentration(compound="z") + simulator.get_total_label_concentration(compound="z"),
+            simulator.get_unlabeled_concentration(compound="z")
+            + simulator.get_total_label_concentration(compound="z"),
             4.0,
         )
 
