@@ -1,7 +1,5 @@
-# Standard Library
 import unittest
 
-# Third party
 import matplotlib.pyplot as plt
 from modelbase.ode import Model, Simulator
 
@@ -197,7 +195,9 @@ class PlottingTests(unittest.TestCase):
         plt.close()
 
     def test_plot_flux_selection_against_variable(self):
-        fig, ax = s.plot_flux_selection_against_variable(rate_names=["vanilla_reaction"], variable="rate_time")
+        fig, ax = s.plot_flux_selection_against_variable(
+            rate_names=["vanilla_reaction"], variable="rate_time"
+        )
         plt.close()
 
     def test_plot_fluxes_against_variable(self):
