@@ -1,7 +1,6 @@
-# Standard Library
+
 import unittest
 
-# Third party
 from modelbase.ode import Model
 from modelbase.ode import ratelaws as rl
 
@@ -168,7 +167,9 @@ class RateLawTests(unittest.TestCase):
         self.assertEqual(rate["reversible"], True)
 
     def test_reversible_mass_action_2_2(self):
-        ratelaw = rl.ReversibleMassAction(substrates=["x1", "x2"], products=["y1", "y2"], k_fwd="k1", k_bwd="k2")
+        ratelaw = rl.ReversibleMassAction(
+            substrates=["x1", "x2"], products=["y1", "y2"], k_fwd="k1", k_bwd="k2"
+        )
         self.assertEqual(ratelaw.get_sbml_function_string(), "k1 * x1 * x2 - k2 * y1 * y2")
         function = ratelaw.get_rate_function()
         self.assertEqual(function.__name__, "reversible_mass_action_2_2")
